# pyplet
# -*- coding: ISO-8859-15 -*-
#
# pyplet - A simple IRC bot who speaks !
#
# (c) 2003 Jerome Alet <alet@librelogiciel.com>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either ersion 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: pypletbot.py,v 1.15 2003/11/05 19:56:12 jalet Exp $
#
# $Log: pypletbot.py,v $
# Revision 1.15  2003/11/05 19:56:12  jalet
# Single, double and anti quotes are not replaced anymore due to
# understandability concerns
#
# Revision 1.14  2003/11/05 19:36:50  jalet
# Special characters are now replaced with their names.
#
# Revision 1.13  2003/11/04 18:07:04  jalet
# Clean exit on Ctrl+C
#
# Revision 1.12  2003/11/04 17:55:38  jalet
# Added antiquote escaping.
#
# Revision 1.11  2003/11/04 14:47:28  jalet
# Now should answer to version command too.
#
# Revision 1.10  2003/11/04 14:27:25  jalet
# Badly placed parenthesis for gettext
#
# Revision 1.9  2003/10/30 08:42:00  jalet
# Added detection of duplicate nickname
#
# Revision 1.8  2003/10/29 15:33:40  jalet
# Dying message
#
# Revision 1.7  2003/10/29 11:31:35  jalet
# Arbitrary limit set to 32 chars for a word
#
# Revision 1.6  2003/10/29 11:22:17  jalet
# No limit
#
# Revision 1.5  2003/10/29 11:03:28  jalet
# Sends notice message to aknowledge the die command.
#
# Revision 1.4  2003/10/29 10:48:00  jalet
# Now logs die message as well
#
# Revision 1.3  2003/10/29 10:42:59  jalet
# Old names
#
# Revision 1.2  2003/10/29 10:15:08  jalet
# Version 1.0 is out.
#
# Revision 1.1  2003/10/29 09:10:16  jalet
# Initial import into CVS
#
#
#

import sys
import os
import getopt
import gettext
import locale
import syslog

from ircbot import SingleServerIRCBot
from irclib import nm_to_n, nm_to_h, irc_lower, ip_numstr_to_quad, ip_quad_to_numstr

from pyplet import version

MAXWORDLEN = 32

class PyPletBotError(Exception):
    """An exception for pyplet related stuff."""
    def __init__(self, message = ""):
        self.message = message
        Exception.__init__(self, message)
    def __repr__(self):
        return self.message
    __str__ = __repr__

class PyPletIRCBot(SingleServerIRCBot):
    def __init__(self, parent, server, port, channel, nickname, realname, ttsengine, all, prefix):
        self.nickcount = 0
        self.shutup = 0    
        self.parent = parent
        self.channel = channel
        self.all = all
        self.prefix = prefix or ""
        self.ttsengine = ttsengine
        SingleServerIRCBot.__init__(self, [(server, port)], nickname, realname)

    def on_nicknameinuse(self, connection, event) :
        self.parent.log_message(_("Nickname %s is already in use, please use --nickname command line option.") % event.arguments()[0], "error")
        sys.exit(-1)
        
    def on_welcome(self, connection, event):
        connection.join(self.channel)

    def on_privmsg(self, connection, event):
        self.do_command(event, event.arguments()[0].split())

    def on_pubmsg(self, connection, event):
        a = [ p.strip() for p in event.arguments()[0].split(":", 1)]
        if len(a) > 1 and irc_lower(a[0]) == irc_lower(self.connection.get_nickname()):
            self.do_command(event, a[1].split())
        elif self.all :    
            self.do_command(event, a)
        
    def do_command(self, event, args):
        global MAXWORDLEN
        
        mynick = self._nickname
        nick = nm_to_n(event.source())
        command = args[0].lower()
        arguments = " ".join(args[1:])
        if command == "shutup" :    
            self.shutup = 1
            message = _("PyPlet : shutting up.")
            self.connection.notice(nick, message)
            self.parent.log_message(message)
        elif command == "speak" :    
            self.shutup = 0
            message = _("PyPlet : now speaking.")
            self.connection.notice(nick, message)
            self.parent.log_message(message)
        elif command == "help" :    
            self.connection.notice(nick, _("I'm an IRC bot. Type '/msg %s yourmessage' to have yourmessage being send to the soundcard via speech synthesis. Available commands : help, version, shutup, speak, die.") % mynick)
        elif command == "version" :    
            self.connection.notice(nick, "%s" % version.__version__)
        elif command == "die":
            message = _("%s killed me !") % nick
            self.parent.log_message(message)
            self.connection.notice(nick, _("You killed me !"))
            self.die(message)
        else :
            if not self.shutup :
                arguments = " ".join(args)
                if arguments :    
                    self.parent.log_message("%s : %s" % (nick, arguments))
                    
                    # handles the optional prefix
                    nb = self.prefix.count("%s")
                    if not nb :    
                        prefix = self.prefix
                    elif nb == 1 :
                        prefix = self.prefix % nick
                    else :    
                        self.parent.log_message(_("Invalid prefix '%s' ignored.") % self.prefix, level="error")
                        prefix = ""
                    arguments = "%s\n%s" % (prefix, arguments)
                        
                    # first delete dangerous characters
                    # replacing them with their full name does
                    # the job remarkably well
                    arguments = arguments.replace("|", " %s " % _("pipe"))
                    arguments = arguments.replace("&", " %s " % _("ampersand"))
                    arguments = arguments.replace("/", " %s " % _("slash"))
                    arguments = arguments.replace("\t"," %s " %  _("tab"))
                    arguments = arguments.replace("\\", " %s " % _("antislash"))
                    
                    # Replace punctuation too.
                    # It is also more secure because some 
                    # punctuation is dangerous.
                    arguments = arguments.replace(":", " %s " % _("colon"))
                    arguments = arguments.replace(".", " %s " % _("dot"))
                    arguments = arguments.replace(",", " %s " % _("comma"))
                    arguments = arguments.replace(";", " %s " % _("semicolon"))
                    arguments = arguments.replace("?", " %s " % _("interrogation point"))
                    arguments = arguments.replace("!", " %s " % _("exclamation point"))
                    
                    # now try to handle quotes 
                    # arguments = arguments.replace("`", " %s " % _("antiquote"))
                    # arguments = arguments.replace("'", " %s " % _("single quote"))
                    #arguments = arguments.replace('"', " %s " % _("double quote"))
                    arguments = arguments.replace("`", "\\`")
                    arguments = arguments.replace("'", "\\'")
                    arguments = arguments.replace('"', '\\"')
                    arguments = " ".join([a for a in arguments.split() if len(a) <= MAXWORDLEN])
                    
                    # launch TTS engine
                    commandline = '%s "%s"' % (self.ttsengine, arguments)
                    os.system(commandline)
                
class PyPletBot :    
    """Base class for pyplet."""
    def __init__(self, doc="pyplet %s (c) 2003 %s" % (version.__version__, version.__author__)) :
        """Initializes the command line tool."""
        # locale stuff
        try :
            locale.setlocale(locale.LC_ALL, None)
            gettext.install("pyplet")
        except (locale.Error, IOError) :
            gettext.NullTranslations().install()
            sys.stderr.write("pyplet : Error while loading translations\n")
    
        # pyplet specific stuff
        self.documentation = doc
        
    def display_version_and_quit(self) :
        """Displays version number, then exists successfully."""
        print version.__version__
        sys.exit(0)
    
    def display_usage_and_quit(self) :
        """Displays command line usage, then exists successfully."""
        print self.documentation
        sys.exit(0)
        
    def parseCommandline(self, argv, short, long, allownothing=0) :
        """Parses the command line, controlling options."""
        # split options in two lists: those which need an argument, those which don't need any
        withoutarg = []
        witharg = []
        lgs = len(short)
        i = 0
        while i < lgs :
            ii = i + 1
            if (ii < lgs) and (short[ii] == ':') :
                # needs an argument
                witharg.append(short[i])
                ii = ii + 1 # skip the ':'
            else :
                # doesn't need an argument
                withoutarg.append(short[i])
            i = ii
                
        for option in long :
            if option[-1] == '=' :
                # needs an argument
                witharg.append(option[:-1])
            else :
                # doesn't need an argument
                withoutarg.append(option)
        
        # we begin with all possible options unset
        parsed = {}
        for option in withoutarg + witharg :
            parsed[option] = None
        
        # then we parse the command line
        args = []       # to not break if something unexpected happened
        try :
            options, args = getopt.getopt(argv, short, long)
            if options :
                for (o, v) in options :
                    # we skip the '-' chars
                    lgo = len(o)
                    i = 0
                    while (i < lgo) and (o[i] == '-') :
                        i = i + 1
                    o = o[i:]
                    if o in witharg :
                        # needs an argument : set it
                        parsed[o] = v
                    elif o in withoutarg :
                        # doesn't need an argument : boolean
                        parsed[o] = 1
                    else :
                        # should never occur
                        raise PyPletBotError, "Unexpected problem when parsing command line"
            elif (not args) and (not allownothing) and sys.stdin.isatty() : # no option and no argument, we display help if we are a tty
                self.display_usage_and_quit()
        except getopt.error, msg :
            sys.stderr.write("%s\n" % msg)
            sys.stderr.flush()
            self.display_usage_and_quit()
        return (parsed, args)
        
    def log_stderr(self, message, level="info") :
        """Sends the message to the appropriate logging subsystem."""
        sys.stderr.write("%s: PyPlet : %s\n" % (level.upper(), message.strip()))
        sys.stderr.flush()
        
    def log_syslog(self, message, level="info") :
        """Sends the message to syslog."""
        # opening and closing every time is clearly not optimal !
        levels = { "error" : "ERR", "warn": "WARNING", "info": "INFO", "debug": "DEBUG" }
        syslog.openlog("PyPlet", 0, syslog.LOG_DAEMON)
        priority = getattr(syslog, "LOG_%s" % levels.get(level.lower(), "DEBUG").upper(), syslog.LOG_DEBUG)
        syslog.syslog(priority, message)
        syslog.closelog()
    
    def run(self, options) :
        """Runs the bot !"""
        self.log_message = getattr(self, "log_%s" % options["log"])
        serversplit = options["server"].split(":", 1)
        server = serversplit[0]
        if len(serversplit) == 2 :
            try :
                port = int(serversplit[1])
            except ValueError :
                raise PyPletBotError, _("Invalid port %s") % str(serversplit[1])
        else :
            port = 6667
        channel = options["channel"] 
        if not channel.startswith('#') :
            channel = "#%s" % channel
        self.ircbot = PyPletIRCBot(self, server, port, channel, options["nickname"], options["realname"], options["tts"], options["all"], options["prefix"])
        try :
            self.ircbot.start()
        except KeyboardInterrupt :    
            self.ircbot.die(_("Stopped at user request."))
            raise
        
