#! /usr/bin/env python
# -*- coding: ISO-8859-15 -*-
#
# pyplet - A simple IRC bot who speaks !
#
# (c) 2003 Jerome Alet <alet@librelogiciel.com>
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# $Id: setup.py,v 1.2 2003/11/04 14:40:51 jalet Exp $
#
# $Log: setup.py,v $
# Revision 1.2  2003/11/04 14:40:51  jalet
# Version 1.20 is out with French translation
#
# Revision 1.1  2003/10/29 09:10:16  jalet
# Initial import into CVS
#
#

import sys
import glob
import os
import shutil
try :
    from distutils.core import setup
except ImportError, msg :    
    sys.stderr.write("%s\n" % msg)
    sys.stderr.write("You need the DistUtils Python module.\nunder Debian, you may have to install the python-dev package.\nOf course, YMMV.\n")
    sys.exit(-1)

sys.path.insert(0, "pyplet")
from pyplet.version import __version__, __doc__

ACTION_CONTINUE = 0
ACTION_ABORT = 1

def checkModule(module) :
    """Checks if a Python module is available or not."""
    try :
        exec "import %s" % module
    except ImportError :    
        return 0
    else :    
        return 1
        
def checkCommand(command) :
    """Checks if a command is available or not."""
    input = os.popen("type %s 2>/dev/null" % command)
    result = input.read().strip()
    input.close()
    return result
    
def checkWithPrompt(prompt, module=None, command=None, helper=None) :
    """Tells the user what will be checked, and asks him what to do if something is absent."""
    sys.stdout.write("Checking for %s availability : " % prompt)
    sys.stdout.flush()
    if command is not None :
        result = checkCommand(command)
    elif module is not None :    
        result = checkModule(module)
    if result :    
        sys.stdout.write("OK\n")
        return ACTION_CONTINUE
    else :    
        sys.stdout.write("NO.\n")
        sys.stderr.write("ERROR : %s not available !\n" % prompt)
        if helper is not None :
            sys.stdout.write("%s\n" % helper)
            sys.stdout.write("You may continue safely if you don't need this functionnality.\n")
        answer = raw_input("%s is missing. Do you want to continue anyway (y/N) ? " % prompt)
        if answer[0:1].upper() == 'Y' :
            return ACTION_CONTINUE
        else :
            return ACTION_ABORT
    
if ("install" in sys.argv) and not ("help" in sys.argv) :
    # checks if Python version is correct, we need >= 2.1
    if not (sys.version > "2.1") :
        sys.stderr.write("pyplet needs at least Python v2.1 !\nYour version seems to be older than that, please update.\nAborted !\n")
        sys.exit(-1)
        
    # checks if some needed Python modules are there or not.
    modulestocheck = [ ("Python-IRClib", "irclib", "Python-IRClib v0.4.1 or above is mandatory and can be found at http://python-irclib.sourceforge.net"), \
                     ]
    commandstocheck = [("Mbrola", "mbrola", "Mbrola may be needed. Mbrola is not Free Software, you're on your own to find it, sorry."), ("Mbrdico", "mbrdico", "Mbrdico may be needed, you can find it on http://tcts.fpms.ac.be/synthesis/mbrdico/"), ("Festival", "festival", "Festival may be needed, you can find it at http://www.cstr.ed.ac.uk/projects/festival/download.html")]
    for (name, module, helper) in modulestocheck :
        action = checkWithPrompt(name, module=module, helper=helper)
        if action == ACTION_ABORT :
            sys.stderr.write("Aborted !\n")
            sys.exit(-1)
            
    # checks if some software are there or not.
    for (name, command, helper) in commandstocheck :
        action = checkWithPrompt(name, command=command, helper=helper)
        if action == ACTION_ABORT :
            sys.stderr.write("Aborted !\n")
            sys.exit(-1)
            
data_files = []
mofiles = glob.glob(os.sep.join(["po", "*", "*.mo"]))
for mofile in mofiles :
    lang = mofile.split(os.sep)[1]
    directory = os.sep.join(["share", "locale", lang, "LC_MESSAGES"])
    data_files.append((directory, [ mofile ]))
    
directory = os.sep.join(["share", "man", "man1"])
manpages = glob.glob(os.sep.join(["man", "*.1"]))    
data_files.append((directory, manpages))

setup(name = "pyplet", version = __version__,
      license = "GNU GPL",
      description = __doc__,
      author = "Jerome Alet",
      author_email = "alet@librelogiciel.com",
      url = "http://www.librelogiciel.com/software/",
      packages = [ "pyplet" ],
      scripts = [ "bin/pyplet", "bin/pyplet-festival.sh", "bin/pyplet-mbrola.sh" ],
      data_files = data_files)

